@@ -1,5 +1,10 @@
+struct InvalidColoringError <: Exception end
+
 """
-    partial_distance2_coloring(bg::BipartiteGraph, ::Val{side}, vertices_in_order::AbstractVector)
+    partial_distance2_coloring(
+        bg::BipartiteGraph, ::Val{side}, vertices_in_order::AbstractVector;
+        forced_colors::Union{AbstractVector{<:Integer},Nothing}=nothing
+    )
 
 Compute a distance-2 coloring of the given `side` (`1` or `2`) in the bipartite graph `bg` and return a vector of integer colors.
 
@@ -7,6 +12,8 @@ A _distance-2 coloring_ is such that two vertices have different colors if they
 
 The vertices are colored in a greedy fashion, following the order supplied.
 
+The optional `forced_colors` keyword argument is used to enforce predefined vertex colors (e.g. coming from another optimization algorithm) but still run the distance-2 coloring procedure to verify correctness.
+
 # See also
 
 - [`BipartiteGraph`](@ref)
@@ -17,11 +24,16 @@ The vertices are colored in a greedy fashion, following the order supplied.
 > [_What Color Is Your Jacobian? Graph Coloring for Computing Derivatives_](https://epubs.siam.org/doi/10.1137/S0036144504444711), Gebremedhin et al. (2005), Algorithm 3.2
 """
 function partial_distance2_coloring(
-    bg::BipartiteGraph{T}, ::Val{side}, vertices_in_order::AbstractVector{<:Integer}
+    bg::BipartiteGraph{T},
+    ::Val{side},
+    vertices_in_order::AbstractVector{<:Integer};
+    forced_colors::Union{AbstractVector{<:Integer},Nothing}=nothing,
 ) where {T,side}
     color = Vector{T}(undef, nb_vertices(bg, Val(side)))
     forbidden_colors = Vector{T}(undef, nb_vertices(bg, Val(side)))
-    partial_distance2_coloring!(color, forbidden_colors, bg, Val(side), vertices_in_order)
+    partial_distance2_coloring!(
+        color, forbidden_colors, bg, Val(side), vertices_in_order; forced_colors
+    )
     return color
 end
 
@@ -30,7 +42,8 @@ function partial_distance2_coloring!(
     forbidden_colors::AbstractVector{<:Integer},
     bg::BipartiteGraph,
     ::Val{side},
-    vertices_in_order::AbstractVector{<:Integer},
+    vertices_in_order::AbstractVector{<:Integer};
+    forced_colors::Union{AbstractVector{<:Integer},Nothing}=nothing,
 ) where {side}
     color .= 0
     forbidden_colors .= 0
@@ -44,17 +57,32 @@ function partial_distance2_coloring!(
                 end
             end
         end
-        for i in eachindex(forbidden_colors)
-            if forbidden_colors[i] != v
-                color[v] = i
-                break
+        if isnothing(forced_colors)
+            for i in eachindex(forbidden_colors)
+                if forbidden_colors[i] != v
+                    color[v] = i
+                    break
+                end
+            end
+        else
+            f = forced_colors[v]
+            if (
+                (f == 0 && length(neighbors(bg, Val(side), v)) > 0) ||
+                (f > 0 && forbidden_colors[f] == v)
+            )
+                throw(InvalidColoringError())
+            else
+                color[v] = f
             end
         end
     end
 end
 
 """
-    star_coloring(g::AdjacencyGraph, vertices_in_order::AbstractVector, postprocessing::Bool)
+    star_coloring(
+        g::AdjacencyGraph, vertices_in_order::AbstractVector, postprocessing::Bool;
+        forced_colors::Union{AbstractVector,Nothing}=nothing
+    )
 
 Compute a star coloring of all vertices in the adjacency graph `g` and return a tuple `(color, star_set)`, where
 
@@ -67,6 +95,8 @@ The vertices are colored in a greedy fashion, following the order supplied.
 
 If `postprocessing=true`, some colors might be replaced with `0` (the "neutral" color) as long as they are not needed during decompression.
 
+The optional `forced_colors` keyword argument is used to enforce predefined vertex colors (e.g. coming from another optimization algorithm) but still run the star coloring procedure to verify correctness and build auxiliary data structures, useful during decompression.
+
 # See also
 
 - [`AdjacencyGraph`](@ref)
@@ -77,7 +107,10 @@ If `postprocessing=true`, some colors might be replaced with `0` (the "neutral"
 > [_New Acyclic and Star Coloring Algorithms with Application to Computing Hessians_](https://epubs.siam.org/doi/abs/10.1137/050639879), Gebremedhin et al. (2007), Algorithm 4.1
 """
 function star_coloring(
-    g::AdjacencyGraph{T}, vertices_in_order::AbstractVector{<:Integer}, postprocessing::Bool
+    g::AdjacencyGraph{T},
+    vertices_in_order::AbstractVector{<:Integer},
+    postprocessing::Bool;
+    forced_colors::Union{AbstractVector{<:Integer},Nothing}=nothing,
 ) where {T<:Integer}
     # Initialize data structures
     nv = nb_vertices(g)
@@ -91,7 +124,7 @@ function star_coloring(
 
     for v in vertices_in_order
         for (w, index_vw) in neighbors_with_edge_indices(g, v)
-            !has_diagonal(g) || (v == w && continue)
+            augmented_graph(g) || (v == w && continue)
             color_w = color[w]
             iszero(color_w) && continue
             forbidden_colors[color_w] = v
@@ -106,7 +139,7 @@ function star_coloring(
             else
                 first_neighbor[color[w]] = (v, w, index_vw)
                 for (x, index_wx) in neighbors_with_edge_indices(g, w)
-                    !has_diagonal(g) || (w == x && continue)
+                    augmented_graph(g) || (w == x && continue)
                     color_x = color[x]
                     (x == v || iszero(color_x)) && continue
                     if x == hub[star[index_wx]]  # potential Case 2 (which is always false for trivial stars with two vertices, since the associated hub is negative)
@@ -115,10 +148,18 @@ function star_coloring(
                 end
             end
         end
-        for i in eachindex(forbidden_colors)
-            if forbidden_colors[i] != v
-                color[v] = i
-                break
+        if isnothing(forced_colors)
+            for i in eachindex(forbidden_colors)
+                if forbidden_colors[i] != v
+                    color[v] = i
+                    break
+                end
+            end
+        else
+            if forbidden_colors[forced_colors[v]] == v  # TODO: handle forced_colors[v] == 0
+                throw(InvalidColoringError())
+            else
+                color[v] = forced_colors[v]
             end
         end
         _update_stars!(star, hub, g, v, color, first_neighbor)
@@ -143,7 +184,7 @@ function _treat!(
     color::AbstractVector{<:Integer},
 )
     for x in neighbors(g, w)
-        !has_diagonal(g) || (w == x && continue)
+        augmented_graph(g) || (w == x && continue)
         color_x = color[x]
         iszero(color_x) && continue
         forbidden_colors[color_x] = v
@@ -163,12 +204,12 @@ function _update_stars!(
     first_neighbor::AbstractVector{<:Tuple},
 )
     for (w, index_vw) in neighbors_with_edge_indices(g, v)
-        !has_diagonal(g) || (v == w && continue)
+        augmented_graph(g) || (v == w && continue)
         color_w = color[w]
         iszero(color_w) && continue
         x_exists = false
         for (x, index_wx) in neighbors_with_edge_indices(g, w)
-            !has_diagonal(g) || (w == x && continue)
+            augmented_graph(g) || (w == x && continue)
             if x != v && color[x] == color[v]  # vw, wx ∈ E
                 star_wx = star[index_wx]
                 hub[star_wx] = w  # this may already be true
@@ -245,16 +286,16 @@ function acyclic_coloring(
 
     for v in vertices_in_order
         for w in neighbors(g, v)
-            !has_diagonal(g) || (v == w && continue)
+            augmented_graph(g) || (v == w && continue)
             color_w = color[w]
             iszero(color_w) && continue
             forbidden_colors[color_w] = v
         end
         for w in neighbors(g, v)
-            !has_diagonal(g) || (v == w && continue)
+            augmented_graph(g) || (v == w && continue)
             iszero(color[w]) && continue
             for (x, index_wx) in neighbors_with_edge_indices(g, w)
-                !has_diagonal(g) || (w == x && continue)
+                augmented_graph(g) || (w == x && continue)
                 color_x = color[x]
                 iszero(color_x) && continue
                 if forbidden_colors[color_x] != v
@@ -271,6 +312,7 @@ function acyclic_coloring(
                 end
             end
         end
+        # TODO: handle forced colors
         for i in eachindex(forbidden_colors)
             if forbidden_colors[i] != v
                 color[v] = i
@@ -278,16 +320,16 @@ function acyclic_coloring(
             end
         end
         for (w, index_vw) in neighbors_with_edge_indices(g, v)  # grow two-colored stars around the vertex v
-            !has_diagonal(g) || (v == w && continue)
+            augmented_graph(g) || (v == w && continue)
             color_w = color[w]
             iszero(color_w) && continue
             _grow_star!(v, w, index_vw, color_w, first_neighbor, forest)
         end
         for (w, index_vw) in neighbors_with_edge_indices(g, v)
-            !has_diagonal(g) || (v == w && continue)
+            augmented_graph(g) || (v == w && continue)
             iszero(color[w]) && continue
             for (x, index_wx) in neighbors_with_edge_indices(g, w)
-                !has_diagonal(g) || (w == x && continue)
+                augmented_graph(g) || (w == x && continue)
                 color_x = color[x]
                 (x == v || iszero(color_x)) && continue
                 if color_x == color[v]
@@ -601,150 +643,3 @@ function TreeSet(
 
     return TreeSet(reverse_bfs_orders, is_star, tree_edge_indices, nt)
 end
-
-## Postprocessing, mirrors decompression code
-
-function postprocess!(
-    color::AbstractVector{<:Integer},
-    star_or_tree_set::Union{StarSet,TreeSet},
-    g::AdjacencyGraph,
-    offsets::AbstractVector{<:Integer},
-)
-    S = pattern(g)
-    edge_to_index = edge_indices(g)
-    # flag which colors are actually used during decompression
-    nb_colors = maximum(color)
-    color_used = zeros(Bool, nb_colors)
-
-    # nonzero diagonal coefficients force the use of their respective color (there can be no neutral colors if the diagonal is fully nonzero)
-    if has_diagonal(g)
-        for i in axes(S, 1)
-            if !iszero(S[i, i])
-                color_used[color[i]] = true
-            end
-        end
-    end
-
-    if star_or_tree_set isa StarSet
-        # only the colors of the hubs are used
-        (; star, hub) = star_or_tree_set
-        nb_trivial_stars = 0
-
-        # Iterate through all non-trivial stars
-        for s in eachindex(hub)
-            h = hub[s]
-            if h > 0
-                color_used[color[h]] = true
-            else
-                nb_trivial_stars += 1
-            end
-        end
-
-        # Process the trivial stars (if any)
-        if nb_trivial_stars > 0
-            rvS = rowvals(S)
-            for j in axes(S, 2)
-                for k in nzrange(S, j)
-                    i = rvS[k]
-                    if i > j
-                        index_ij = edge_to_index[k]
-                        s = star[index_ij]
-                        h = hub[s]
-                        if h < 0
-                            h = abs(h)
-                            spoke = h == j ? i : j
-                            if color_used[color[spoke]]
-                                # Switch the hub and the spoke to possibly avoid adding one more used color
-                                hub[s] = spoke
-                            else
-                                # Keep the current hub
-                                color_used[color[h]] = true
-                            end
-                        end
-                    end
-                end
-            end
-        end
-    else
-        # only the colors of non-leaf vertices are used
-        (; reverse_bfs_orders, is_star, tree_edge_indices, nt) = star_or_tree_set
-        nb_trivial_trees = 0
-
-        # Iterate through all non-trivial trees
-        for k in 1:nt
-            # Position of the first edge in the tree
-            first = tree_edge_indices[k]
-
-            # Total number of edges in the tree
-            ne_tree = tree_edge_indices[k + 1] - first
-
-            # Check if we have more than one edge in the tree (non-trivial tree)
-            if ne_tree > 1
-                # Determine if the tree is a star
-                if is_star[k]
-                    # It is a non-trivial star and only the color of the hub is needed
-                    (_, hub) = reverse_bfs_orders[first]
-                    color_used[color[hub]] = true
-                else
-                    # It is not a star and both colors are needed during the decompression
-                    (i, j) = reverse_bfs_orders[first]
-                    color_used[color[i]] = true
-                    color_used[color[j]] = true
-                end
-            else
-                nb_trivial_trees += 1
-            end
-        end
-
-        # Process the trivial trees (if any)
-        if nb_trivial_trees > 0
-            for k in 1:nt
-                # Position of the first edge in the tree
-                first = tree_edge_indices[k]
-
-                # Total number of edges in the tree
-                ne_tree = tree_edge_indices[k + 1] - first
-
-                # Check if we have exactly one edge in the tree
-                if ne_tree == 1
-                    (i, j) = reverse_bfs_orders[first]
-                    if color_used[color[i]]
-                        # Make i the root to avoid possibly adding one more used color
-                        # Switch it with the (only) leaf
-                        reverse_bfs_orders[first] = (j, i)
-                    else
-                        # Keep j as the root
-                        color_used[color[j]] = true
-                    end
-                end
-            end
-        end
-    end
-
-    # if at least one of the colors is useless, modify the color assignments of vertices
-    if any(!, color_used)
-        num_colors_useless = 0
-
-        # determine what are the useless colors and compute the offsets
-        for ci in 1:nb_colors
-            if color_used[ci]
-                offsets[ci] = num_colors_useless
-            else
-                num_colors_useless += 1
-            end
-        end
-
-        # assign the neutral color to every vertex with a useless color and remap the colors
-        for i in eachindex(color)
-            ci = color[i]
-            if !color_used[ci]
-                # assign the neutral color
-                color[i] = 0
-            else
-                # remap the color to not have any gap
-                color[i] -= offsets[ci]
-            end
-        end
-    end
-    return color
-end
